@@ -1,6 +1,6 @@
 import numpy as np
 from .rollable import RollableFunction
-from .sequences import as_sequence_array, Sequence
+from .sequences import as_encoded_sequence_array, Sequence
 from .encodings import ACTGEncoding
 
 
@@ -13,6 +13,11 @@ class PositionWeightMatrix(RollableFunction):
 
     def __call__(self, sequence: Sequence) -> float:
         if self._encoding is not None:
-            sequence = as_sequence_array(sequence, self._encoding)
+            sequence = as_encoded_sequence_array(sequence, self._encoding)
         scores = self._matrix[np.asarray(sequence), self._indices]
         return scores.sum(axis=-1)
+
+
+def pwm_from_counts(count_matrix):
+    with_pseudo = count_matrix+1
+    return np.log(with_pseudo/with_pseudo.sum(axis=0, keepdims=True))
